
#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <SDL.h>
#if defined(HAVE_SDL_SDL_IMAGE_H)
#include <SDL/SDL_image.h>
#endif

#include "level.h"
#include "trace.h"
#include "image.h"

#define XSIZE 640
#define YSIZE 480


static void Game_Quit(void);
static void Game_Init(int argc, char **argv);
static void Game_SplashScreen(void);
static void Game_Menu_Loop(void);
static int Game_Menu(void);
static int Game_Run(struct t_level *lvl);
static int Game_PlayLevel(void);

SDL_Surface *screen;

struct t_game
{
  int turns;


} game;

int main(int argc, char **argv)
{
  Game_Init(argc,argv);
  Game_SplashScreen();
  Game_Menu_Loop();
  Game_Quit();
  return 0;
}


static void Game_Init(int argc, char **argv)
{
  int videoflags, video_bpp;

  if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
   {
     fprintf(stderr, "Impossible d'initialiser SDL: %s\n", SDL_GetError());
     exit(1);
   }
  atexit(SDL_Quit);

  video_bpp = 0;
  videoflags = SDL_SWSURFACE | SDL_DOUBLEBUF;
  while ( argc > 1 )
   {
     --argc;
     if ( strcmp(argv[argc-1], "-bpp") == 0 )
      {
	video_bpp = atoi(argv[argc]);
	--argc;
      }
     else if ( strcmp(argv[argc], "-warp") == 0 )
      {
	videoflags |= SDL_HWPALETTE;
      }
     else if ( strcmp(argv[argc], "-hw") == 0 )
      {
	videoflags |= SDL_HWSURFACE;
      }
     else if ( strcmp(argv[argc], "-fullscreen") == 0 )
      {
	videoflags |= SDL_FULLSCREEN;
      }
     else
      {
	fprintf(stderr, "Usage: %s [-bpp N] [-warp] [-hw] [-fullscreen]\n", argv[0]);
	exit(1);
      }
   }

  screen = SDL_SetVideoMode(640, 480, video_bpp, videoflags);
  if ( screen == NULL )
   {
     fprintf(stderr, "Impossible de passer en 640x480 en %d bpp: %s\n",video_bpp,SDL_GetError());
     exit(1);
   }

  /* Initialise global game variables */

  game.turns=9;
  
}

/*
 * Function: Game_Menu_Loop
 *
 * Description:
 *   This function manage the result of the main menu. With this menu, the user
 *   can choose to:
 *     * start a new level
 *     * change musik
 *     * quit the game ...
 *     ...
 * Input:
 * Output:
 */
static void Game_Menu_Loop(void)
{
  int action;

  while (1)
   {
     switch(action=Game_Menu())
      {
       case 1:
	 Game_PlayLevel();
	 break;
       case 2:
	 trace("Want to quit");
	 return;
      }
   }
}

/*
 * Function: Game_PlayLevel
 *
 * Description:
 *   This launch the game level specified by (game.turns). When the user
 *   wins a new level is launched. When the user loose, the function draw
 *   a loose splash screen
 *
 * Input:
 * Output:
 *   return a code to indicate if the user have loose,won, ...
 */
static int Game_PlayLevel(void)
{
  struct t_level *lvl;
  char levelname[FILENAME_MAX];
  int action;


  /* Draw interscreen */

  
  while (1)
   {
     /* Draw a new level interscreen */

     
     /* Load Level */
     snprintf(levelname,FILENAME_MAX,"level%2.2d.lvl",game.turns);
     lvl = Level_Load(levelname);
     if (!lvl)
      {
	trace("Can't load the level");
	return 0;
      }
     if (!Level_Sanity_Check(lvl))
      {
	trace("This level is corrupted");
	return 0;
      }

     /* Play Level */
     action = Game_Run(lvl);
     switch (action)
      {
       case 1:
	 /* Player Lose (Timeout) */
	 trace("You lose ... timeout\n");

	 return 0;
	 break;
       case 2:
	 /* Player want to quit */
	 trace("Is it too difficult for you ?\n");

	 return 0;
	 break;
       case 3:
	 /* Player win */
	 trace("Yo, your are a good man\n");

	 game.turns++;
	 break;
       default:
	 trace("unknown Game_Run action %d",action);
      }
   }
}


static void Game_SplashScreen(void)
{
  SDL_Surface *image=NULL;
  SDL_Event event;

  image = Image_Load("images/splash.jpg");
  if (image!=NULL)
   {
     SDL_BlitSurface(image, NULL, screen, NULL);
     SDL_Flip(screen);
     SDL_FreeSurface(image);
     //wait for key or button
     while (1) 
      {
	SDL_WaitEvent(&event);
	if (event.type == SDL_QUIT) 
	  Game_Quit();
	if (event.type == SDL_KEYUP || event.type == SDL_MOUSEBUTTONUP)
	  break;
     }
   }
}

/*
 * Function: Game_Menu
 *
 * Description:
 *   This function is responsible to draw the main menu, and wait an action
 *   choosen by the user. When an action occured, the function returns a code
 *   to indicate this choice.
 *
 * Input:
 * Output:
 *   return a code to indicate if the user have loose,won, ...
 */

static int Game_Menu(void)
{
  int action;
  SDL_Surface *image=NULL;
  SDL_Event event;

  image = Image_Load("images/menu.jpg");
  if (image==NULL)
    Game_Quit();
  SDL_BlitSurface(image, NULL, screen, NULL);
  SDL_Flip(screen);
  SDL_FreeSurface(image);

  action=0;
  while (!action) 
   {
     SDL_WaitEvent(&event);
     if (event.type == SDL_QUIT) 
       Game_Quit();
     if (event.type == SDL_KEYUP)
      {
	switch(event.key.keysym.sym)
	  {
	   case SDLK_F1:
	     action=1;
	     break;
	   case SDLK_q:
	     action=2;
	     break;
	   default:
	     action=0;
	     break;
	  }
      }
   }
  return action;
}
static void Game_Quit(void)
{
  if (screen)
    SDL_FreeSurface(screen);
  SDL_Quit();
  exit(0);
}

struct
{
  int keysym;
  int direction;
//  int (*fct_movedinoto)(struct t_level *l, enum movement_type direction);
//  int (*fct_movekinesistto)(struct t_level *l, enum movement_type direction);;
//  int (*fct_movesquareto)(struct t_level *l, enum movement_type direction);;
} static move_actions[] =
{
   {SDLK_DOWN,MOVE_DOWN},
   {SDLK_UP,MOVE_UP},
   {SDLK_RIGHT,MOVE_RIGHT},
   {SDLK_LEFT,MOVE_LEFT},
   {0,0}
};

static int Game_Run(struct t_level *lvl)
{
  int action,we_have_moved,i;
  SDL_Event event;

  Level_Draw2Screen(lvl,screen);
  SDL_Flip(screen);
  action=0;
  while (!action) 
   {
     SDL_WaitEvent(&event);
     if (event.type == SDL_QUIT) 
       Game_Quit();
     if (event.type == SDL_KEYUP)
      {
	we_have_moved=0;
	switch(event.key.keysym.sym)
	 {
	  case SDLK_q:
	    action=2;
	    break;
	  case SDLK_SPACE: /* Popup a carre to move a square */
	    switch (lvl->control)
	     {
	      case CONTROL_PLAYER:
		/* Now we control the select square */
		lvl->control=CONTROL_KINESIST;
		lvl->xkinesist=lvl->xdino;
		lvl->ykinesist=lvl->ydino;
		break;
	      case CONTROL_KINESIST:
		/* Find if a moveable square is under the carre rouge */
		if (Level_CanMoveSquare(lvl))
		 {
		   /* We test again, because if the block can move again,
		    * the control return to the kinesist */
		   if (Level_CanMoveSquare(lvl))
		     lvl->control=CONTROL_SQUARE;
		   else
		     lvl->control=CONTROL_KINESIST;
		 }
		else
		 {
		   /* Give control to the player */
		   lvl->control=CONTROL_PLAYER;
		 }
		break;
	      case CONTROL_SQUARE:
		lvl->control=CONTROL_PLAYER;

		break;
	     }
	    we_have_moved=1; /* Refresh the screen */
	    break;
	  default:
	    action=0;
	    for (i=0;move_actions[i].keysym != 0; i++)
	     {
	       if (event.key.keysym.sym == move_actions[i].keysym)
		{
		  switch (lvl->control)
		   {
		    case CONTROL_PLAYER:
		      we_have_moved=Level_MoveDinoTo(lvl,move_actions[i].direction);
		      break;
		    case CONTROL_KINESIST:
		      we_have_moved=Level_MoveKinesistTo(lvl,move_actions[i].direction);
		      break;
		    case CONTROL_SQUARE:
		      we_have_moved=Level_MoveSquareTo(lvl,move_actions[i].direction);
		      break;
		   }
		  break; /* Exit the for () */
		}
	     }
	 } /* End of switch(event.key.keysym.sym) */
	if (we_have_moved)
	 {
	   Level_DoPostMoveAction(lvl);
	   Level_Draw2Screen(lvl,screen);
	   SDL_Flip(screen);
	   if (lvl->control==CONTROL_PLAYER && Level_IsFinish(lvl))
	     action=3;
	 }
      }
   }
  return action;
}
